@@ -1,11 +1,12 @@
 # $Header: /home/jesse/DBIx-SearchBuilder/history/SearchBuilder/Handle.pm,v 1.21 2002/01/28 06:11:37 jesse Exp $
 package DBIx::SearchBuilder::Handle;
+use strict;
 use Carp;
 use DBI;
-use strict;
 use Class::ReturnValue;
-use vars qw($VERSION @ISA %DBIHandle $PrevHandle $DEBUG $TRANSDEPTH);
+use Encode;
 
+use vars qw($VERSION @ISA %DBIHandle $PrevHandle $DEBUG $TRANSDEPTH);
 
 $TRANSDEPTH = 0;
 
@@ -22,24 +23,16 @@ DBIx::SearchBuilder::Handle - Perl extension which is a generic DBI handle
 
   use DBIx::SearchBuilder::Handle;
 
- my $Handle = DBIx::SearchBuilder::Handle->new();
- $Handle->Connect( Driver => 'mysql',
-		   Database => 'dbname',
-		   Host => 'hostname',
-		   User => 'dbuser',
-		   Password => 'dbpassword');
+  my $handle = DBIx::SearchBuilder::Handle->new();
+  $handle->Connect( Driver => 'mysql',
+                    Database => 'dbname',
+                    Host => 'hostname',
+                    User => 'dbuser',
+                    Password => 'dbpassword');
  
- 
-
 =head1 DESCRIPTION
 
-Jesse's a slacker.
-
-Blah blah blah.
-
-=head1 AUTHOR
-
-Jesse Vincent, jesse@fsck.com
+This class provides a wrapper for DBI handles that can also perform a number of additional functions.
  
 =cut
 
@@ -65,35 +58,6 @@ sub new  {
 
 # }}}
 
-# {{{ sub Insert
-=head2 Insert $TABLE_NAME @KEY_VALUE_PAIRS
-
-Takes a table name and a set of key-value pairs in an array. splits the key value pairs, constructs an INSERT statement and performs the insert. Returns the row_id of this row.
-
-=cut
-
-sub Insert {
-  my($self, $table, @pairs) = @_;
-  my(@cols, @vals, @bind);
-
-#  my %seen; #only the *first* value is used - allows drivers to specify default
-  while ( my $key = shift @pairs ) {
-    my $value = shift @pairs;
-    #    next if $seen{$key}++;
-    push @cols, $key;
-    push @vals, '?';
-    push @bind, $value;  
-  }
-
-  my $QueryString =
-    "INSERT INTO $table (". join(", ", @cols). ") VALUES ".
-    "(". join(", ", @vals). ")";
-
-    my $sth =  $self->SimpleQuery($QueryString, @bind);
-    return ($sth);
-  }
-# }}}
-
 # {{{ sub Connect 
 
 =head2 Connect PARAMHASH: Driver, Database, Host, User, Password
@@ -121,7 +85,7 @@ sub Connect  {
            DisconnectHandleOnDestroy => undef,
 	       @_);
 
-    my $dsn = $self->DSN;
+    my $dsn = $self->DSN || '';
 
     # Setting this actually breaks old RT versions in subtle ways. So we need to explicitly call it
 
@@ -358,6 +322,35 @@ sub dbh {
 
 # }}}
 
+# {{{ sub Insert
+=head2 Insert $TABLE_NAME @KEY_VALUE_PAIRS
+
+Takes a table name and a set of key-value pairs in an array. splits the key value pairs, constructs an INSERT statement and performs the insert. Returns the row_id of this row.
+
+=cut
+
+sub Insert {
+  my($self, $table, @pairs) = @_;
+  my(@cols, @vals, @bind);
+
+  #my %seen; #only the *first* value is used - allows drivers to specify default
+  while ( my $key = shift @pairs ) {
+    my $value = shift @pairs;
+    # next if $seen{$key}++;
+    push @cols, $key;
+    push @vals, '?';
+    push @bind, $value;  
+  }
+
+  my $QueryString =
+    "INSERT INTO $table (". join(", ", @cols). ") VALUES ".
+    "(". join(", ", @vals). ")";
+
+    my $sth =  $self->SimpleQuery($QueryString, @bind);
+    return ($sth);
+  }
+# }}}
+
 # {{{ sub UpdateRecordValue 
 
 =head2 UpdateRecordValue 
@@ -472,6 +465,9 @@ sub SimpleQuery {
             delete $bhash->{'value'};
             $sth->bind_param( $bind_idx + 1, undef, $bhash );
         }
+        # Some databases, such as Oracle fail to cope if it's a perl utf8
+        # string. they desperately want bytes.
+         Encode::_utf8_off($bind_values[$bind_idx]);
     }
 
     my $basetime;
@@ -812,6 +808,7 @@ sub Join {
         TABLE2        => undef,
         FIELD2        => undef,
         ALIAS2        => undef,
+        EXPRESSION    => undef,
         @_
     );
 
@@ -886,12 +883,21 @@ sub Join {
         $string = " JOIN " . $args{'TABLE2'} . " $alias ";
 
     }
+
+
+    my $criterion;
+    if ($args{'EXPRESSION'}) {
+        $criterion = $args{'EXPRESSION'};
+    } else {
+        $criterion = $args{'ALIAS1'}.".".$args{'FIELD1'};
+    }
+
     $args{'SearchBuilder'}->{'left_joins'}{"$alias"}{'alias_string'} = $string;
     $args{'SearchBuilder'}->{'left_joins'}{"$alias"}{'depends_on'}   =
       $args{'ALIAS1'};
     $args{'SearchBuilder'}->{'left_joins'}{"$alias"}{'criteria'}
       { 'criterion' . $args{'SearchBuilder'}->{'criteria_count'}++ } =
-      " $args{'ALIAS1'}.$args{'FIELD1'} = $alias.$args{'FIELD2'}";
+      " $alias.$args{'FIELD2'} = $criterion";
 
     return ($alias);
 }
@@ -1061,6 +1067,10 @@ __END__
 
 # {{{ POD
 
+=head1 AUTHOR
+
+Jesse Vincent, jesse@fsck.com
+
 =head1 SEE ALSO
 
 perl(1), L<DBIx::SearchBuilder>
